function varx = varianza_una_va(xpart,xval)
%  varx = varianza_una_va(xpart,xval)
%  Dada una variable aleatoria en forma particional, hallar su varianza
%  La variable esta definida sobre el espacio muestral [0,1],
%     con una distribucion uniforme
%  xpart: vector (fila o columna) de puntos que definen la particion
%         1er elemento: 0, ultimo elemento: 1, orden ascendiente
%  xval:  valores correspondientes a los distintos elementos de la particion
%  varx: varianza de la variable aleatoria
% 
%  Ejemplo:  xpart = [ 0 1/4 1/3 1/2 3/4 1 ]
%            xval =  [  10  15  20  15  10 ]
%  El output es: varx = 13.889

%% Funcion de Matlab/Octave -- Ricard Torres, 2015 (sin acentos)

% En primer lugar, comprobamos que los argumentos sean correctos
if ( nargin ~= 2 )
   mostrar_uso() ;
   error('La funcion debe tener dos argumentos') ;
end

if ~ and( isvector(xpart), isvector(xval) )
   mostrar_uso() ;
   error('Los argumentos de la funcion deben de ser vectores') ;
end

lp = length(xpart); lv = length(xval);

if ~ ( lv == lp - 1 )
   mostrar_uso() ;
   error('Debe de haber un valor para cada elemento de la particion') ;
end

if ~ and( (xpart(1) == 0), (xpart(end) == 1) )
   mostrar_uso() ;
   error('La particion debe empezar con 0 y acabar con 1') ;
end

is_ascending = true;
for i = 2 : lp
    if xpart(i) <= xpart(i-1), is_ascending = false; end
end

if ~ is_ascending
   mostrar_uso() ;
   error('El vector que define la particion debe tener orden ascendiente') ;
end

% Ahora desarrollamos los calculos propiamente dichos
[ distx, valx ] = distrib_una_va(xpart,xval) ;
lx = length(valx);
% Asegurar que es vector fila
valx = reshape(valx, 1, lx);
media = dot(distx, valx) ;
xcentrada = valx - media * ones(1, lx);
varx = dot(distx, xcentrada.^2);

end

% Mensaje de ayuda en caso de error
function mostrar_uso()
fprintf('Uso: varx = varianza_una_va(xpart,xval)\n');
fprintf('Dada una variable aleatoria en forma particional, ');
fprintf('hallar su varianza\n');
fprintf('La variable esta definida sobre el espacio muestral [0,1],\n');
fprintf('  con una distribucion uniforme\n');
fprintf('xpart: vector (fila o columna) de puntos que definen la particion\n');
fprintf('    1er elemento: 0, ultimo elemento: 1, orden ascendiente\n');
fprintf('xval:  valores correspondientes a los distintos elementos ');
fprintf(' de la particion\n');
fprintf('varx: varianza de la variable aleatoria\n');
fprintf('\nEjemplo:  xpart = [ 0 1/4 1/3 1/2 3/4 1 ]\n');
fprintf('          xval =  [  10  15  20  15  10 ]\n');
fprintf('El output es: varx = 13.889\n');
end

